/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2007, 2012. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.recordgen.cobol;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

/**
 * An object that represents the OCCURS clause associated with a COBOL COPYBOOK
 * symbol.  This class represents a fixed number of elements.
 */
public class Occurs {

	private int max;

	/**
	 * Construct and instance
	 * @param max
	 */
	public Occurs(int max) {
		this.max = max;
	}

	/**
	 * Return the number of elements that OCCUR
	 * @return int
	 */
	public int getMax() {
		return max;
	}

	/**
	 * @see #getMax()
	 */
	public void setMax(int max) {
		this.max = max;
	}
	
	/**
	 * A debug representation of the receiver.
	 */
	public String toString() {
		return "Occurs: " + max;
	}

	/**
	 * Write the receiver in XML form
	 * @since 2.4.4
	 */
	public void writeXmlAttributesOn(XMLStreamWriter writer) throws XMLStreamException {
		writer.writeAttribute("occurs", Integer.toString(getMax()));		
	}
	
	/**
	 * Parse an instance of Occurs or a subtype from XML
	 * @since 2.4.4
	 */
	public static Occurs parseXmlFrom(RecordSpecTreeXMLParser parser) throws XMLStreamException {
		Occurs answer = null;
		int occurs = parser.getAttributeIntValue("occurs", -1);
		if (occurs >= 0) {
			answer = new Occurs(occurs);
		} else {
			int occursMin = parser.getAttributeIntValue("occursMin", -1);
			if (occursMin >= 0) {
				answer = OccursDependingOn.parseXmlFrom(parser);
			}
		}
		return answer;
	}
	
	/**
	 * Parse XML attributes
	 * @since 2.4.4
	 */
	protected int parseAttributeIntValue(XMLStreamReader reader, String name, int def) {
		String v = reader.getAttributeValue(null, name);
		if (v == null) {
			return def;
		}
		return Integer.parseInt(v);
	}
	
}
