/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2007, 2012. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.recordgen.cobol;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 * An object that represents the offset of an ElementSpec from the beginning of the record.
 * <p>This class represents a fixed offset from the beginning of the record.
 */
public class Offset implements Cloneable {
	int offset;

	/**
	 * Construct an instance
	 * @param offset
	 */
	public Offset(int offset) {
		this.offset = offset;
	}

	/**
	 * Return a clone of the receiver.
	 */
	public Object clone() {
		return new Offset(offset);
	}

	/**
	 * Return the fixed offset
	 * @return int
	 */
	public int getOffset() {
		return offset;
	}
	
	/**
	 * Increase the offset by increment
	 * @param increment
	 */
	public void incrementOffset(int increment) {
		offset += increment;
	}
	
	/**
	 * @see #getOffset()
	 * @param offset
	 */
	public void setOffset(int offset) {
		this.offset = offset;
	}
	
	/**
	 * A debug representation of the receiver.
	 */
	public String toString() {
		return "+" + offset;
	}
	
	/**
	 * Write the receiver in XML form
	 * @since 2.4.4
	 */
	public void writeXmlAttributesOn(XMLStreamWriter writer) throws XMLStreamException {
		writer.writeAttribute("offset", Integer.toString(getOffset()));
	}
	
	/**
	 * Parse and create/return an Offset from XML
	 * @since 2.4.4
	 */
	public static Offset parseXmlFrom(RecordSpecTreeXMLParser parser) throws XMLStreamException {
		int offset = parser.getAttributeIntValue("offset", -1);
		if (offset < 0) return null;
		int offsetDependingOn = parser.getAttributeIntValue("offsetDependingOn", -1);
		if (offsetDependingOn < 0) {
			return new Offset(offset);
		} else {
			ElementSpec spec = parser.getSpecById(offsetDependingOn);
			if (!(spec instanceof StorageSpec)) {
				throw new XMLStreamException("offsetDependingOn invalid id", parser.getReader().getLocation());
			}			
			return new VariableOffset(offset, (StorageSpec)spec);
		}
	}
}
